@@ -563,8 +563,12 @@ class WorkTracker:
         self.__uptime_period = NOT_STAKING
         self._abort_on_error = False
 
+        self._consecutive_fails = 0
+
     @classmethod
-    def random_interval(cls) -> int:
+    def random_interval(cls, fails=None) -> int:
+        if fails is not None and fails > 0:
+            return cls.INTERVAL_FLOOR
         return random.randint(cls.INTERVAL_FLOOR, cls.INTERVAL_CEIL)
 
     @property
@@ -572,7 +576,7 @@ class WorkTracker:
         return self.__current_period
 
     def max_confirmation_time(self) -> int:
-        expected_time = EXPECTED_CONFIRMATION_TIME_IN_SECONDS[self.gas_strategy]
+        expected_time = EXPECTED_CONFIRMATION_TIME_IN_SECONDS[self.gas_strategy]  # FIXME: #2447
         result = expected_time * (1 + self.ALLOWED_DEVIATION)
         return result
 
@@ -581,7 +585,7 @@ class WorkTracker:
             self._tracking_task.stop()
             self.log.info(f"STOPPED WORK TRACKING")
 
-    def start(self, act_now: bool = True, requirement_func: Callable = None, force: bool = False) -> None:
+    def start(self, commit_now: bool = True, requirement_func: Callable = None, force: bool = False) -> None:
         """
         High-level stake tracking initialization, this function aims
         to be safely called at any time - For example, it is okay to call
@@ -598,8 +602,8 @@ class WorkTracker:
         self.__uptime_period = self.staking_agent.get_current_period()
         self.__current_period = self.__uptime_period
 
-        self.log.info(f"START WORK TRACKING")
-        d = self._tracking_task.start(interval=self.random_interval(), now=act_now)
+        self.log.info(f"START WORK TRACKING (immediate action: {commit_now})")
+        d = self._tracking_task.start(interval=self.random_interval(fails=self._consecutive_fails), now=commit_now)
         d.addErrback(self.handle_working_errors)
 
     def _crash_gracefully(self, failure=None) -> None:
@@ -618,9 +622,16 @@ class WorkTracker:
             self.stop()
             reactor.callFromThread(self._crash_gracefully, failure=failure)
         else:
-            self.log.warn(f'Unhandled error during work tracking: {failure.getTraceback()!r}',
+            self.log.warn(f'Unhandled error during work tracking (#{self._consecutive_fails}): {failure.getTraceback()!r}',
                           failure=failure)
-            self.start()
+
+            # the effect of this is that we get one immediate retry.
+            # After that, the random_interval will be honored until
+            # success is achieved
+            commit_now = self._consecutive_fails < 1
+            self._consecutive_fails += 1
+            self.start(commit_now=commit_now)
+
 
     def __work_requirement_is_satisfied(self) -> bool:
         # TODO: Check for stake expiration and exit
@@ -640,8 +651,10 @@ class WorkTracker:
         tx_count_pending = self.client.get_transaction_count(account=worker_address, pending=True)
         tx_count_latest = self.client.get_transaction_count(account=worker_address, pending=False)
         txs_in_mempool = tx_count_pending - tx_count_latest
+
         if len(self.__pending) == txs_in_mempool:
             return True  # OK!
+
         if txs_in_mempool > len(self.__pending):  # We're missing some pending TXs
             return False
         else:  # TODO #2429: What to do when txs_in_mempool < len(self.__pending)? What does this imply?
@@ -650,13 +663,17 @@ class WorkTracker:
     def __track_pending_commitments(self) -> bool:
         # TODO: Keep a purpose-built persistent log of worker transaction history
 
-        unmined_transactions = list()
+        unmined_transactions = 0
         pending_transactions = self.pending.items()    # note: this must be performed non-mutatively
         for tx_firing_block_number, txhash in sorted(pending_transactions):
+            if txhash is UNTRACKED_PENDING_TRANSACTION:
+                unmined_transactions += 1
+                continue
+
             try:
                 confirmed_tx_receipt = self.client.get_transaction_receipt(transaction_hash=txhash)
             except TransactionNotFound:
-                unmined_transactions.append(txhash)  # mark as unmined - Keep tracking it for now
+                unmined_transactions += 1  # mark as unmined - Keep tracking it for now
                 continue
             else:
                 confirmation_block_number = confirmed_tx_receipt['blockNumber']
@@ -665,8 +682,8 @@ class WorkTracker:
                 del self.__pending[tx_firing_block_number]
 
         if unmined_transactions:
-            s = "s" if len(unmined_transactions) > 1 else ""
-            self.log.info(f'{len(unmined_transactions)} pending commitment transaction{s} detected.')
+            s = "s" if unmined_transactions > 1 else ""
+            self.log.info(f'{unmined_transactions} pending commitment transaction{s} detected.')
 
         inconsistent_tracker = not self.__commitments_tracker_is_consistent()
         if inconsistent_tracker:
@@ -705,6 +722,10 @@ class WorkTracker:
         self.__fire_replacement_commitment(current_block_number=current_block_number,
                                            tx_firing_block_number=tx_firing_block_number)
 
+    def __reset_tracker_state(self) -> None:
+        self.__pending.clear()  # Forget the past. This is a new beginning.
+        self._consecutive_fails = 0
+
     def _do_work(self) -> None:
         """
         Async working task for Ursula  # TODO: Split into multiple async tasks
@@ -718,6 +739,8 @@ class WorkTracker:
         onchain_period = self.staking_agent.get_current_period()  # < -- Read from contract
         if self.current_period != onchain_period:
             self.__current_period = onchain_period
+            self.log.info(f"New period is {self.__current_period}")
+            self.__reset_tracker_state()
 
             # TODO: #1515 and #1517 - Shut down at end of terminal stake
             # This slows down tests substantially and adds additional
@@ -727,6 +750,7 @@ class WorkTracker:
         # Measure working interval
         interval = onchain_period - self.worker.last_committed_period
         if interval < 0:
+            self.__reset_tracker_state()
             return  # No need to commit to this period.  Save the gas.
         if interval > 0:
             # TODO: #1516 Follow-up actions for missed commitments
@@ -741,7 +765,7 @@ class WorkTracker:
             return  # This cycle is finished.
         else:
             # Randomize the next task interval over time, within bounds.
-            self._tracking_task.interval = self.random_interval()
+            self._tracking_task.interval = self.random_interval(fails=self._consecutive_fails)
 
         # Only perform work this round if the requirements are met
         if not self.__work_requirement_is_satisfied():
