@@ -14,6 +14,8 @@
  You should have received a copy of the GNU Affero General Public License
  along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 """
+
+
 from decimal import Decimal
 
 import click
@@ -35,7 +37,8 @@ from nucypher.cli.actions.confirm import (
     confirm_enable_restaking_lock,
     confirm_enable_winding_down,
     confirm_large_stake,
-    confirm_staged_stake, confirm_disable_snapshots
+    confirm_staged_stake,
+    confirm_disable_snapshots
 )
 from nucypher.cli.actions.select import select_client_account_for_staking, select_stake
 from nucypher.cli.config import group_general_config, GroupGeneralConfig
@@ -67,13 +70,32 @@ from nucypher.cli.literature import (
     SUCCESSFUL_SET_MIN_POLICY_RATE,
     SUCCESSFUL_STAKE_DIVIDE,
     SUCCESSFUL_STAKE_PROLONG,
-    SUCCESSFUL_WORKER_BONDING, NO_MINTABLE_PERIODS, STILL_LOCKED_TOKENS, CONFIRM_MINTING, SUCCESSFUL_MINTING,
-    CONFIRM_COLLECTING_WITHOUT_MINTING, NO_TOKENS_TO_WITHDRAW, NO_FEE_TO_WITHDRAW, CONFIRM_INCREASING_STAKE,
-    PROMPT_STAKE_INCREASE_VALUE, SUCCESSFUL_STAKE_INCREASE, INSUFFICIENT_BALANCE_TO_INCREASE, MAXIMUM_STAKE_REACHED,
-    INSUFFICIENT_BALANCE_TO_CREATE, PROMPT_STAKE_CREATE_VALUE, PROMPT_STAKE_CREATE_LOCK_PERIODS,
-    ONLY_DISPLAYING_MERGEABLE_STAKES_NOTE, CONFIRM_MERGE, SUCCESSFUL_STAKES_MERGE, SUCCESSFUL_ENABLE_SNAPSHOTS,
-    SUCCESSFUL_DISABLE_SNAPSHOTS, CONFIRM_ENABLE_SNAPSHOTS,
-    CONFIRM_STAKE_USE_UNLOCKED, CONFIRM_REMOVE_SUBSTAKE, SUCCESSFUL_STAKE_REMOVAL)
+    SUCCESSFUL_WORKER_BONDING,
+    NO_MINTABLE_PERIODS,
+    STILL_LOCKED_TOKENS,
+    CONFIRM_MINTING,
+    SUCCESSFUL_MINTING,
+    CONFIRM_COLLECTING_WITHOUT_MINTING,
+    NO_TOKENS_TO_WITHDRAW,
+    NO_FEE_TO_WITHDRAW,
+    CONFIRM_INCREASING_STAKE,
+    PROMPT_STAKE_INCREASE_VALUE,
+    SUCCESSFUL_STAKE_INCREASE,
+    INSUFFICIENT_BALANCE_TO_INCREASE,
+    MAXIMUM_STAKE_REACHED,
+    INSUFFICIENT_BALANCE_TO_CREATE,
+    PROMPT_STAKE_CREATE_VALUE,
+    PROMPT_STAKE_CREATE_LOCK_PERIODS,
+    ONLY_DISPLAYING_MERGEABLE_STAKES_NOTE,
+    CONFIRM_MERGE,
+    SUCCESSFUL_STAKES_MERGE,
+    SUCCESSFUL_ENABLE_SNAPSHOTS,
+    SUCCESSFUL_DISABLE_SNAPSHOTS,
+    CONFIRM_ENABLE_SNAPSHOTS,
+    CONFIRM_STAKE_USE_UNLOCKED,
+    CONFIRM_REMOVE_SUBSTAKE,
+    SUCCESSFUL_STAKE_REMOVAL
+)
 from nucypher.cli.options import (
     group_options,
     option_config_file,
@@ -102,7 +124,8 @@ from nucypher.cli.types import (
     EIP55_CHECKSUM_ADDRESS,
     EXISTING_READABLE_FILE,
     GWEI,
-    DecimalRange)
+    DecimalRange
+)
 from nucypher.cli.utils import setup_emitter
 from nucypher.config.characters import StakeHolderConfiguration
 from nucypher.utilities.gas_strategies import construct_fixed_price_gas_strategy
@@ -138,7 +161,6 @@ class StakeHolderConfigOptions:
                 signer_uri=self.signer_uri,
                 poa=self.poa,
                 light=self.light,
-                sync=False,
                 domain=self.network,
                 registry_filepath=self.registry_filepath)
 
@@ -166,7 +188,6 @@ class StakeHolderConfigOptions:
             signer_uri=self.signer_uri,
             poa=self.poa,
             light=self.light,
-            sync=False,
             registry_filepath=self.registry_filepath,
             domain=self.network
         )
@@ -275,7 +296,7 @@ class TransactingStakerOptions:
         blockchain = self.staker_options.get_blockchain()
         if self.gas_price:  # TODO: Consider performing this step in the init of EthereumClient
             fixed_price_strategy = construct_fixed_price_gas_strategy(gas_price=self.gas_price, denomination="gwei")
-            blockchain.set_gas_strategy(fixed_price_strategy)
+            blockchain.configure_gas_strategy(fixed_price_strategy)
         return blockchain
 
 
@@ -549,6 +570,7 @@ def create(general_config: GroupGeneralConfig,
     if not force:
         confirm_large_stake(value=value, lock_periods=lock_periods)
         paint_staged_stake(emitter=emitter,
+                           blockchain=blockchain,
                            stakeholder=STAKEHOLDER,
                            staking_address=staking_address,
                            stake_value=value,
@@ -645,6 +667,7 @@ def increase(general_config: GroupGeneralConfig,
 
         confirm_large_stake(value=value, lock_periods=lock_periods)
         paint_staged_stake(emitter=emitter,
+                           blockchain=blockchain,
                            stakeholder=STAKEHOLDER,
                            staking_address=staking_address,
                            stake_value=value,
@@ -906,6 +929,7 @@ def divide(general_config: GroupGeneralConfig,
     if not force:
         confirm_large_stake(lock_periods=extension, value=value)
         paint_staged_stake_division(emitter=emitter,
+                                    blockchain=blockchain,
                                     stakeholder=STAKEHOLDER,
                                     original_stake=current_stake,
                                     target_value=value,
