@@ -113,56 +113,58 @@ def _make_rest_app(datastore: Datastore, this_node, domain: str, log: Logger) ->
 
     @rest_app.route("/public_information")
     def public_information():
-        """
-        REST endpoint for public keys and address.
-        """
-        response = Response(
-            response=bytes(this_node),
-            mimetype='application/octet-stream')
-
+        """REST endpoint for public keys and address."""
+        response = Response(response=bytes(this_node), mimetype='application/octet-stream')
         return response
 
-    @rest_app.route("/ping", methods=['POST'])
+    @rest_app.route("/ping", methods=['GET', 'POST'])
     def ping():
         """
-        Asks this node: "Can you access my public information endpoint"?
+        GET: Asks this node: "What is my IP address?"
+        POST: Asks this node: "Can you access my public information endpoint?"
         """
 
-        try:
-            requesting_ursula = Ursula.from_bytes(request.data)
-            requesting_ursula.mature()
-        except ValueError:  # (ValueError)
-            return Response({'error': 'Invalid Ursula'}, status=400)
-        else:
-            initiator_address, initiator_port = tuple(requesting_ursula.rest_interface)
+        if request.method == 'GET':
+            requester_ip_address = request.remote_addr
+            return Response(requester_ip_address, status=200)
 
-        # Compare requester and posted Ursula information
-        request_address = request.environ['REMOTE_ADDR']
-        if request_address != initiator_address:
-            return Response({'error': 'Suspicious origin address'}, status=400)
+        elif request.method == 'POST':
+            try:
+                requesting_ursula = Ursula.from_bytes(request.data)
+                requesting_ursula.mature()
+            except ValueError:
+                return Response({'error': 'Invalid Ursula'}, status=400)
+            else:
+                initiator_address, initiator_port = tuple(requesting_ursula.rest_interface)
 
-        #
-        # Make a Sandwich
-        #
+            # Compare requester and posted Ursula information
+            request_address = request.remote_addr
+            if request_address != initiator_address:
+                message = f'Origin address mismatch: Request origin is {request_address} but metadata claims {initiator_address}.'
+                return Response({'error': message}, status=400)
 
-        try:
-            # Fetch and store initiator's teacher certificate.
-            certificate = this_node.network_middleware.get_certificate(host=initiator_address, port=initiator_port)
-            certificate_filepath = this_node.node_storage.store_node_certificate(certificate=certificate)
-            requesting_ursula_bytes = this_node.network_middleware.client.node_information(host=initiator_address,
-                                                                                           port=initiator_port,
-                                                                                           certificate_filepath=certificate_filepath)
-        except NodeSeemsToBeDown:
-            return Response({'error': 'Unreachable node'}, status=400)  # ... toasted
+            #
+            # Make a Sandwich
+            #
 
-        except InvalidNodeCertificate:
-            return Response({'error': 'Invalid TLS certificate - missing checksum address'}, status=400)  # ... invalid
+            try:
+                # Fetch and store initiator's teacher certificate.
+                certificate = this_node.network_middleware.get_certificate(host=initiator_address, port=initiator_port)
+                certificate_filepath = this_node.node_storage.store_node_certificate(certificate=certificate)
+                requesting_ursula_bytes = this_node.network_middleware.client.node_information(host=initiator_address,
+                                                                                               port=initiator_port,
+                                                                                               certificate_filepath=certificate_filepath)
+            except NodeSeemsToBeDown:
+                return Response({'error': 'Unreachable node'}, status=400)  # ... toasted
 
-        # Compare the results of the outer POST with the inner GET... yum
-        if requesting_ursula_bytes == request.data:
-            return Response(status=200)
-        else:
-            return Response({'error': 'Suspicious node'}, status=400)
+            except InvalidNodeCertificate:
+                return Response({'error': 'Invalid TLS certificate - missing checksum address'}, status=400)  # ... invalid
+
+            # Compare the results of the outer POST with the inner GET... yum
+            if requesting_ursula_bytes == request.data:
+                return Response(status=200)
+            else:
+                return Response({'error': 'Suspicious node'}, status=400)
 
     @rest_app.route('/node_metadata', methods=["GET"])
     def all_known_nodes():
@@ -207,7 +209,7 @@ def _make_rest_app(datastore: Datastore, this_node, domain: str, log: Logger) ->
         with datastore.describe(PolicyArrangement, arrangement.id.hex(), writeable=True) as new_policy_arrangement:
             new_policy_arrangement.arrangement_id = arrangement.id.hex().encode()
             new_policy_arrangement.expiration = arrangement.expiration
-            new_policy_arrangement.alice_verifying_key = arrangement.alice.stamp.as_umbral_pubkey()
+            new_policy_arrangement.alice_verifying_key = arrangement.alice_verifying_key
 
         # TODO: Fine, we'll add the arrangement here, but if we never hear from Alice again to enact it,
         # we need to prune it at some point.  #1700
@@ -421,7 +423,7 @@ def _make_rest_app(datastore: Datastore, this_node, domain: str, log: Logger) ->
     @rest_app.route('/status/', methods=['GET'])
     def status():
         if request.args.get('json'):
-            payload = this_node.abridged_node_details()
+            payload = this_node.abridged_node_details(raise_invalid=False)
             response = jsonify(payload)
             return response
 
